@@ -1,4 +1,22 @@
-"""A learning method to align the intensity ranges of images."""
+# Copyright (C) 2013 Oskar Maier
+# 
+# This program is free software: you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+# 
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+# 
+# You should have received a copy of the GNU General Public License
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+#
+# author Oskar Maier
+# version r0.1.2
+# since 2013-09-04
+# status Release
 
 # build-in modules
 
@@ -10,19 +28,12 @@ from scipy.interpolate.interpolate import interp1d
 
 # own modules
 
-# information
-__author__ = "Oskar Maier"
-__version__ = "r0.1, 2013-09-04"
-__email__ = "oskar.maier@googlemail.com"
-__status__ = "Release"
-__description__ = "A learning method to align the intensity ranges of images."
-
 # code
 class IntensityRangeStandardization (object):
-    """
+    r"""
     Class to standardize intensity ranges between a number of images.
     
-    Short description:
+    **Short description:**
     Often images containing similar objects or scenes have different intensity ranges
     that make it difficult to compare them manually as well as to process them
     further.
@@ -32,37 +43,9 @@ class IntensityRangeStandardization (object):
     information using a multi-segment linear transformation model.
     
     Once learned, this model can be applied to other, formerly unseen images to map
-    them to the same standard intensity space.
-    
-    Application example:
-    We have a number of similar images with varying intensity ranges. To make them
-    comparable, we would like to transform them to a common intensity space. Thus we
-    run:
-    
-        from medpy.filter import IntensityRangeStandardization
-        irs = IntensityRangeStandardization()
-        trained_model, transformed_images = irs.train_transform(images)
-        
-    Let us assume we now obtain another, new image, that we would like to make
-    comparable to the others. As long as it does not differ to much from these, we
-    can simply call:
-        
-        transformed_image = irs.transform(new_image)
-        
-    For many application, not all images are already available at the time of
-    execution. It would therefore be good to be able to preserve a once trained
-    model. The solution is to just pickle the once trained model:
-    
-        import pickle
-        with open('my_trained_model.pkl', 'wb') as f:
-            pickle.dump(irs, f)
+    them to the same standard intensity space.    
             
-    And load it again when required with:
-    
-        with open('my_trained_model.pkl', 'r') as f:
-            irs = pickle.load(f)
-            
-    Concept of similar images:
+    **Concept of similar images:**
     IntensityRangeStandardization is limited to similar images. Images containing
     different object or different compositions of objects are not suitable to be
     transformed to a common intensity space (and it would furthermore not make much
@@ -73,7 +56,7 @@ class IntensityRangeStandardization (object):
     from the same patient and using the same scanner. For further processing, e.g.
     for training a classifier, they have to be mapped to a common intensity space.
     
-    Failure of the transformation:
+    **Failure of the transformation:**
     The method implemented in IntensityRangeStandardization ensures that no
     information is lost i.e. a lossless transformation is performed. This can be
     assured when there exists a one-to-one mapping between the images original
@@ -90,22 +73,23 @@ class IntensityRangeStandardization (object):
     to a new intensity standard space, all already transformed images have to be
     processed again.
     
-    Setting the training parameters:
+    **Setting the training parameters:**
     The method comes with a set of default parameters, that are suitable for most
     cases. But for some special cases, it might be better to set them on your own. Ti
     understand the working of the parameters, it is recommended to read the detailed
     method description first.
     
-    The method depends of three parameters:
+    **The method depends on three parameters:**
+    
     cutoffp, i.e. the cut-off percentiles
         These are used to the define the intensity outliers, both during training and
         image transformation. The default values are usualy a good choice.
-        (in [1] these are called the minimum and maximum percentile values pc1 and pc2 respectively)
+        (in [1]_ these are called the minimum and maximum percentile values pc1 and pc2 respectively)
     landmarkp, i.e. the landmark percentiles
         These percentiles define the landmark positions. The more supplied, the more
         exact but less general becomes the model. It is common to supply equally
         spaced percentiles between 0 and 100.
-        (in [1] these are called the landmark locations mu_1, .., mu_l)
+        (in [1]_ these are called the landmark locations mu_1, .., mu_l)
     strange, i.e. the standard intensity space range
         These two intensity values define roughly the standard intensity space (or
         common intensity space of the images; or even target intensity space) to
@@ -114,12 +98,12 @@ class IntensityRangeStandardization (object):
         training process. It is additionally possible to supply only the lower or
         upper range border and set the other to ''auto'', in which case the method
         chooses the range automatically, but not the position. 
-        (in [1] these are called the minimum and maximum intensities on the standard scale of the IOI s1 resp. s2)
+        (in [1]_ these are called the minimum and maximum intensities on the standard scale of the IOI s1 resp. s2)
     
     
-    Details of the method:
+    **Details of the method:**
     In the following the method is described in some more detail. For even more
-    information see [1].
+    information see [1]_.
          
     Essentially the method is based on a multi-segment linear transformation model. A
     standard intensity space (or common intensity space) is defined by an intensity
@@ -144,34 +128,66 @@ class IntensityRangeStandardization (object):
     last segmented of the transformation model. Not that this means the transformed
     images intensity values do not always lie inside the standard intensity space
     range, but are fitted as best as possible inside.
+         
+    Parameters
+    ----------
+    cutoffp : (float, float)
+        Lower and upper cut-off percentiles to exclude outliers.
+    landmarkp : sequence of floats
+        List of percentiles serving as model landmarks, must lie
+        between the cutoffp values.
+    stdrange : string or (float, float)
+        The range of the standard intensity space for which a
+        transformation is learned; when set to 'auto, automatically
+        determined from the training image upon training; it is also
+        possible to fix either the upper or the lower border value and
+        setting the other to 'auto'.
+        
+    Examples
+    --------
+    We have a number of similar images with varying intensity ranges. To make them
+    comparable, we would like to transform them to a common intensity space. Thus we
+    run:
     
-    The implementation is based on:
-    [1] Nyul, L.G.; Udupa, J.K.; Xuan Zhang, "New variants of a method of MRI scale
-        standardization," Medical Imaging, IEEE Transactions on , vol.19, no.2, pp.143-150,
-        Feb. 2000
-    """
+        >>> from medpy.filter import IntensityRangeStandardization
+        >>> irs = IntensityRangeStandardization()
+        >>> trained_model, transformed_images = irs.train_transform(images)
+        
+    Let us assume we now obtain another, new image, that we would like to make
+    comparable to the others. As long as it does not differ to much from these, we
+    can simply call:
+        
+        >>> transformed_image = irs.transform(new_image)
+        
+    For many application, not all images are already available at the time of
+    execution. It would therefore be good to be able to preserve a once trained
+    model. The solution is to just pickle the once trained model:
+    
+        >>> import pickle
+        >>> with open('my_trained_model.pkl', 'wb') as f:
+        >>>     pickle.dump(irs, f)
+            
+    And load it again when required with:
+    
+        >>> with open('my_trained_model.pkl', 'r') as f:
+        >>>     irs = pickle.load(f)
+        
+    References
+    ----------
+    .. [1] Nyul, L.G.; Udupa, J.K.; Xuan Zhang, "New variants of a method of MRI scale
+       standardization," Medical Imaging, IEEE Transactions on , vol.19, no.2, pp.143-150,
+       Feb. 2000
+    """    
     
     # static member variables
     L2 = [50]
+    """1-value landmark points model."""
     L3 = [25, 50, 75]
+    """3-value landmark points model."""
     L4 = [10, 20, 30, 40, 50, 60, 70, 80, 90]
+    """9-value landmark points model."""
     
     def __init__(self, cutoffp = (1, 99), landmarkp = L4, stdrange = 'auto'):
-        """
-        See class description for more details.
-        
-        @param cutoffp lower and upper cut-off percentiles to exclude outliers
-        @type cutoffp 2-tuple of numbers
-        @param landmarkp list of percentiles serving as model landmarks, must lie between
-                         the cutoffp values
-        @type landmarkp sequence of numbers
-        @param stdrange the range of the standrad intensity space for which a
-                        transformation is learned; when set to ''auto'', automatically
-                        determined from the training image upon training; it is also
-                        possible to fix either the upper or the lower border value and
-                        setting the other to ''auto'' 
-        @type stdrange string | 2-tuple of numbers
-        """
         # check parameters
         if not IntensityRangeStandardization.is_sequence(cutoffp):
             raise ValueError('cutoffp must be a sequence')
@@ -226,16 +242,20 @@ class IntensityRangeStandardization (object):
         self.__sc_umaxs = None
         
     def train(self, images):
-        """
+        r"""
         Train a standard intensity space and an associated transformation model.
         
         Note that the passed images should be masked to contain only the foreground.
         
-        @param images a number of images
-        @type image sequence of ndarrays
+        Parameters
+        ----------
+        images : sequence of array_likes
+            A number of images.
         
-        @return this instance of IntensityRangeStandardization
-        @rtype IntensityRangeStandardization
+        Returns
+        -------
+        IntensityRangeStandardization : IntensityRangeStandardization
+            This instance of IntensityRangeStandardization
         """
         self.__stdrange = self.__compute_stdrange(images)
         
@@ -257,26 +277,34 @@ class IntensityRangeStandardization (object):
         return self
         
     def transform(self, image, surpress_mapping_check = False):
-        """
+        r"""
         Transform an images intensity values to the learned standard intensity space.
         
         Note that the passed image should be masked to contain only the foreground.
         
         The transformation is guaranteed to be lossless i.e. a one-to-one mapping between
         old and new intensity values exists. In cases where this does not hold, an error
-        is thrown. This can be suppressed by setting surpress_mapping_check to ''True''.
+        is thrown. This can be suppressed by setting ``surpress_mapping_check`` to 'True'.
         Do this only if you know what you are doing.
         
-        @param image the image to transform
-        @type image ndarray
-        @param surpress_mapping_check whether to ensure a lossless transformation or not
-        @type surpress_mapping_check bool
+        Parameters
+        ----------
+        image : array_like
+            The image to transform.
+        surpress_mapping_check : bool
+            Whether to ensure a lossless transformation or not.
         
-        @return the transformed image
-        @rtype ndarray
+        Returns
+        -------
+        image : ndarray
+            The transformed image
         
-        @raises InformationLossException if a lossless transformation can not be ensured
-        @raises Exception if no model has been trained before
+        Raises
+        -------
+        InformationLossException 
+            If a lossless transformation can not be ensured
+        Exception
+            If no model has been trained before
         """
         if None == self.__model:
             raise UntrainedException('Model not trained. Call train() first.')
@@ -309,6 +337,11 @@ class IntensityRangeStandardization (object):
         return output
     
     def train_transform(self, images, surpress_mapping_check = False):
+        r"""
+        See also
+        --------
+        train, transform
+        """
         ret = self.train(images)
         outputs = [self.transform(i, surpress_mapping_check) for i in images]
         return ret, outputs
@@ -330,11 +363,11 @@ class IntensityRangeStandardization (object):
     
     @property
     def model(self):
-        """Get the model i.e. the learned percentile values."""
+        """Get the model (the learned percentile values)."""
         return self.__model
     
     def __compute_stdrange(self, images):
-        """
+        r"""
         Computes a common standard intensity range over a number of images.
         
         Depending on the settings of the internal self.__stdrange variable,
@@ -348,31 +381,41 @@ class IntensityRangeStandardization (object):
         all segments over all images.
         
         Be
-        \f[
+        
+        .. math::
+        
             L = (cop_l, lp_1, lp_2, ..., lp_n, cop_u)
-        \f]
-        the set formed by the two cut-off percentiles \f$cop_l\f$ and \f$cop_u\f$ and the
-        landmark percentiles \f$lp_1, ..., lp_n\f$. The corresponding intensity values of
-        an image \f$i\in I\f$ are then
-        \f[
+        
+        the set formed by the two cut-off percentiles :math:`cop_l` and :math:`cop_u` and the
+        landmark percentiles :math:`lp_1, ..., lp_n`. The corresponding intensity values of
+        an image :math:`i\in I` are then
+        
+        .. math::
+            
             V_i = (v_{i,1}, v_{i,2}, ..., v_{i,n+2})
-        \f]
+        
         The distance between each of these intensity values forms a segment along the
-        images \f$i\f$ intensity range denoted as
-        \f[
+        images :math:`i` intensity range denoted as
+        
+        ..math ::
+            
             S_i = (s_{i,1}, s_{i,2}, ..., s_{i, n+1})
-        \f]
-        The common standard intensity range \f$sir\f$ over the set of images \f$I\f$ is
+        
+        The common standard intensity range :math:`sir` over the set of images :math:`I` is
         then defined as
-        \f[
+        
+        ..math ::
             sir = \sum_{l=1}^{n+1}\max_{i=1}^I s_{i,l} * \max_{l=1}^{n+1} \left(\frac{\max_{i=1}^I s_{i,l}}{\min_{i=1}^I s_{i,l}}\right)
-        \f]
         
-        @param images a number of images
-        @type images sequence of ndarrays
-        
-        @return the borders of the computed standard intensity range
-        @rtype tuple of numbers
+        Parameters
+        ----------
+        images : sequence of array_like
+            A number of images.
+            
+        Returns
+        -------
+        stdrange : (float, float)
+            The borders of the computed standard intensity range.
         """
         if not 'auto' in self.__stdrange:
             return self.__stdrange
@@ -482,7 +525,7 @@ class IntensityRangeStandardization (object):
         """
         Returns a linear model transformation function fitted on the two supplied points.
         y = m*x + b
-        Note: Assumes that slope > 0, otherwise division through zero might occure.
+        Note: Assumes that slope > 0, otherwise division through zero might occur.
         """
         m = (y2 - y1) / (x2 - x1)
         b = y1 - (m * x1)
